@@ -5,7 +5,7 @@
 # pylint: disable=cyclic-import
 # pyright: reportUnusedImport=false
 
-__version__ = "0.26.0"
+__version__ = "0.30.0"
 
 import sys
 import warnings
@@ -14,16 +14,7 @@ from .command import AuditEventInfo, CmdContext, Command, Options
 from .pipeline import Pipeline
 from .pty_util import cbreak, cooked, raw
 from .result import Result, ResultError
-from .runner import (
-    AUDIT_EVENT_SUBPROCESS_SPAWN,
-    UNLAUNCHED_EXIT_CODE,
-    PipeRunner,
-    Runner,
-)
-
-if sys.platform != "win32":
-    from .watcher import DefaultChildWatcher
-
+from .runner import PipeRunner, Runner
 
 if sys.version_info[:3] in [(3, 10, 9), (3, 11, 1)]:
     # Warn about these specific Python releases: 3.10.9 and 3.11.1
@@ -60,8 +51,5 @@ __all__ = [
     "ResultError",
     "Runner",
     "PipeRunner",
-    "DefaultChildWatcher",
-    "AUDIT_EVENT_SUBPROCESS_SPAWN",
     "AuditEventInfo",
-    "UNLAUNCHED_EXIT_CODE",
 ]
