@@ -6,12 +6,13 @@
 """
 
 import asyncio
+import collections.abc
 import dataclasses
 import enum
+import io
 import os
 import shutil
 import signal
-from collections.abc import Iterable
 from dataclasses import dataclass, field
 from pathlib import Path
 from types import TracebackType
@@ -24,6 +25,7 @@ from typing import (
     Coroutine,
     Generator,
     Generic,
+    Iterable,
     Optional,
     Sequence,
     TypedDict,
@@ -47,9 +49,9 @@ from shellous.runner import Runner
 from shellous.util import EnvironmentDict, context_aenter, context_aexit
 
 
-# Sentinel used in "mergable" keyword arguments to indicate that a value
-# was not set by the caller. This is an enum class to make UNSET more visible
-# in generated documentation.
+# Sentinel used in keyword arguments to indicate that a value was not set by
+# the caller. This is an enum class to make UNSET more visible in generated
+# documentation.
 class _UnsetEnum(enum.Enum):
     UNSET = enum.auto()
 
@@ -65,7 +67,10 @@ _PreexecFnT = Optional[Callable[[], None]]
 
 
 class AuditEventInfo(TypedDict):
-    "Info attached to each Audit Event."
+    """Info attached to each audit callback event.
+
+    See `audit_callback` in `Command.set` for more information.
+    """
 
     runner: Runner
     "Reference to the Runner object."
@@ -78,6 +83,7 @@ class AuditEventInfo(TypedDict):
 
 
 _AuditFnT = Optional[Callable[[str, AuditEventInfo], None]]
+_CoerceArgFnT = Optional[Callable[[Any], Any]]
 
 
 @dataclass(frozen=True)
@@ -118,7 +124,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
     "True if error object should be closed after subprocess launch."
 
     encoding: str = "utf-8"
-    "Specifies encoding of input/ouput."
+    "Specifies encoding of input/output."
 
     _return_result: bool = False
     "True if we should return `Result` object instead of the output text/bytes."
@@ -151,7 +157,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
     "True if using process substitution in write mode."
 
     _start_new_session: bool = False
-    "True if child process should start a new session with setsid()."
+    "True if child process should start a new session with `setsid` call."
 
     _preexec_fn: _PreexecFnT = None
     "Function to call in child process after fork from parent."
@@ -165,8 +171,11 @@ class Options:  # pylint: disable=too-many-instance-attributes
     audit_callback: _AuditFnT = None
     "Function called to audit stages of process execution."
 
-    def merge_env(self) -> Optional[dict[str, str]]:
-        "Return our `env` merged with the global environment."
+    coerce_arg: _CoerceArgFnT = None
+    "Function called to coerce top level arguments."
+
+    def runtime_env(self) -> Optional[dict[str, str]]:
+        "@private Return our `env` merged with the global environment."
         if self.inherit_env:
             if not self.env:
                 return None
@@ -177,7 +186,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
         return {}
 
     def set_stdin(self, input_: Any, close: bool) -> "Options":
-        "Return new options with `input` configured."
+        "@private Return new options with `input` configured."
         if input_ is None:
             raise TypeError("invalid stdin")
 
@@ -191,7 +200,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
         )
 
     def set_stdout(self, output: Any, append: bool, close: bool) -> "Options":
-        "Return new options with `output` configured."
+        "@private Return new options with `output` configured."
         if output is None:
             raise TypeError("invalid stdout")
 
@@ -206,7 +215,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
         )
 
     def set_stderr(self, error: Any, append: bool, close: bool) -> "Options":
-        "Return new options with `error` configured."
+        "@private Return new options with `error` configured."
         if error is None:
             raise TypeError("invalid stderr")
 
@@ -217,29 +226,40 @@ class Options:  # pylint: disable=too-many-instance-attributes
             error_close=close,
         )
 
-    def set_env(self, updates: dict[str, Any]) -> "Options":
-        "Return new options with augmented environment."
+    def add_env(self, updates: dict[str, Any]) -> "Options":
+        "@private Return new options with augmented environment."
         new_env = EnvironmentDict(self.env, updates)
         return dataclasses.replace(self, env=new_env)
 
     def set(self, kwds: dict[str, Any]) -> "Options":
-        """Return new options with given properties updated.
+        """@private Return new options with given properties updated.
 
         See `Command.set` for option reference.
         """
         kwds = {key: value for key, value in kwds.items() if value is not _UNSET}
+        if "env" in kwds:
+            # The "env" property is stored as an `EnvironmentDict`.
+            new_env = kwds["env"]
+            if new_env:
+                kwds["env"] = EnvironmentDict(None, new_env)
+            else:
+                kwds["env"] = None
         return dataclasses.replace(self, **kwds)
 
     @overload
     def which(self, name: bytes) -> Optional[bytes]:
-        "Find the command with the given name and return its path."
+        "@private Find the command with the given name and return its path."
 
     @overload
-    def which(self, name: str) -> Optional[str]:
-        "Find the command with the given name and return its path."
+    def which(
+        self, name: Union[str, os.PathLike[Any]]
+    ) -> Optional[Union[str, os.PathLike[Any]]]:
+        "@private Find the command with the given name and return its path."
 
-    def which(self, name: Union[str, bytes]) -> Optional[Union[str, bytes]]:
-        "Find the command with the given name and return its path."
+    def which(
+        self, name: Union[str, bytes, os.PathLike[Any]]
+    ) -> Optional[Union[str, bytes, os.PathLike[Any]]]:
+        "@private Find the command with the given name and return its path."
         return shutil.which(name, path=self.path)
 
 
@@ -303,13 +323,14 @@ class CmdContext(Generic[_RT]):
 
     def env(self, **kwds: Any) -> "CmdContext[_RT]":
         """Return new context with augmented environment."""
-        new_options = self.options.set_env(kwds)
+        new_options = self.options.add_env(kwds)
         return CmdContext(new_options)
 
     def set(  # pylint: disable=unused-argument, too-many-locals
         self,
         *,
         path: Unset[Optional[str]] = _UNSET,
+        env: Unset[dict[str, Any]] = _UNSET,
         inherit_env: Unset[bool] = _UNSET,
         encoding: Unset[str] = _UNSET,
         _return_result: Unset[bool] = _UNSET,
@@ -327,6 +348,7 @@ class CmdContext(Generic[_RT]):
         pty: Unset[PtyAdapterOrBool] = _UNSET,
         close_fds: Unset[bool] = _UNSET,
         audit_callback: Unset[_AuditFnT] = _UNSET,
+        coerce_arg: Unset[_CoerceArgFnT] = _UNSET,
     ) -> "CmdContext[_RT]":
         """Return new context with custom options set.
 
@@ -340,7 +362,7 @@ class CmdContext(Generic[_RT]):
 
     def __call__(self, *args: Any) -> "Command[_RT]":
         "Construct a new command."
-        return Command(coerce(args), self.options)
+        return Command(coerce(args, self.options.coerce_arg), self.options)
 
     @property
     def result(self) -> "CmdContext[shellous.Result]":
@@ -387,7 +409,7 @@ class Command(Generic[_RT]):
     ```
     """
 
-    args: "tuple[Union[str, bytes, Command[Any], shellous.Pipeline[Any]], ...]"
+    args: "tuple[Union[str, bytes, os.PathLike[Any], Command[Any], shellous.Pipeline[Any]], ...]"
     "Command arguments including the program name as first argument."
 
     options: Options
@@ -440,14 +462,22 @@ class Command(Generic[_RT]):
         return Command(self.args, new_options)
 
     def env(self, **kwds: Any) -> "Command[_RT]":
-        """Return new command with augmented environment."""
-        new_options = self.options.set_env(kwds)
+        """Return new command with augmented environment.
+
+        The changes to the environment variables made by this method are
+        additive. For example, calling `cmd.env(A=1).env(B=2)` produces a
+        command with the environment set to `{"A": "1", "B": "2"}`.
+
+        To clear the environment, use the `cmd.set(env={})` method.
+        """
+        new_options = self.options.add_env(kwds)
         return Command(self.args, new_options)
 
     def set(  # pylint: disable=unused-argument, too-many-locals
         self,
         *,
         path: Unset[Optional[str]] = _UNSET,
+        env: Unset[dict[str, Any]] = _UNSET,
         inherit_env: Unset[bool] = _UNSET,
         encoding: Unset[str] = _UNSET,
         _return_result: Unset[bool] = _UNSET,
@@ -465,14 +495,28 @@ class Command(Generic[_RT]):
         pty: Unset[PtyAdapterOrBool] = _UNSET,
         close_fds: Unset[bool] = _UNSET,
         audit_callback: Unset[_AuditFnT] = _UNSET,
+        coerce_arg: Unset[_CoerceArgFnT] = _UNSET,
     ) -> "Command[_RT]":
         """Return new command with custom options set.
 
+        **path** (str | None) default=None<br>
+        Search path for locating command executable. By default, `path` is None
+        which causes shellous to rely on the `PATH` environment variable.
+
+        **env** (dict[str, str]) default={}<br>
+        Set the environment variables for the subprocess. If `inherit_env` is
+        True, the subprocess will also inherit the environment variables
+        specified by the parent process.
+
+        Using `set(env=...)` will replace all environment variables using the
+        dictionary argument. You can also use the `env(...)` method to modify
+        the existing environment incrementally.
+
         **inherit_env** (bool) default=True<br>
         Subprocess should inherit the parent process environment. If this is
         False, the subprocess will only have environment variables specified
         by `Command.env`. If `inherit_env` is True, the parent process
-        environment is augmented/overriden by any variables specified in
+        environment is augmented/overridden by any variables specified in
         `Command.env`.
 
         **encoding** (str) default="utf-8"<br>
@@ -576,6 +620,12 @@ class Command(Generic[_RT]):
         The primary use case for `audit_callback` is measuring how long each
         command takes to run and exporting this information to a metrics
         framework like Prometheus.
+
+        **coerce_arg** (Callable(arg) | None) default=None<br>
+        Specify a function to call on each command line argument. This function
+        can specify how to coerce unsupported argument types (e.g. dict) to
+        a sequence of strings. This function should return the original value
+        unchanged if there is no conversion needed.
         """
         kwargs = locals()
         del kwargs["self"]
@@ -610,7 +660,7 @@ class Command(Generic[_RT]):
 
     async def __aenter__(self) -> Runner:
         "Enter the async context manager."
-        return await context_aenter(id(self), Runner(self))
+        return await context_aenter(self, Runner(self))
 
     async def __aexit__(
         self,
@@ -619,7 +669,7 @@ class Command(Generic[_RT]):
         exc_tb: Optional[TracebackType],
     ) -> Optional[bool]:
         "Exit the async context manager."
-        return await context_aexit(id(self), exc_type, exc_value, exc_tb)
+        return await context_aexit(self, exc_type, exc_value, exc_tb)
 
     def __aiter__(self) -> AsyncIterator[str]:
         "Return async iterator to iterate over output lines."
@@ -635,7 +685,8 @@ class Command(Generic[_RT]):
         "Apply more arguments to the end of the command."
         if not args:
             return self
-        return Command(self.args + coerce(args), self.options)
+        new_args = self.args + coerce(args, self.options.coerce_arg)
+        return Command(new_args, self.options)
 
     def __str__(self) -> str:
         """Return string representation for command.
@@ -694,23 +745,40 @@ class Command(Generic[_RT]):
         )
 
 
-def coerce(args: Sequence[Any]) -> tuple[Any, ...]:
-    """Flatten lists and coerce arguments to string."""
+def coerce(args: Iterable[Any], coerce_arg: _CoerceArgFnT) -> tuple[Any, ...]:
+    """Flatten lists and coerce arguments to string/bytes.
+
+    This function flattens sequence types. It leaves `Iterable` types alone
+    because some types like `IPv4Network` support iteration.
+    """
     result: list[Any] = []
     for arg in args:
-        if isinstance(
-            arg,
-            (str, bytes, bytearray, os.PathLike, Command, shellous.Pipeline),
-        ):
+        if coerce_arg is not None:
+            arg = coerce_arg(arg)
+
+        if isinstance(arg, (str, bytes, os.PathLike, Command, shellous.Pipeline)):
             result.append(arg)
-        elif isinstance(arg, (list, tuple)):
-            result.extend(coerce(arg))  # pyright: ignore[reportUnknownArgumentType]
-        elif isinstance(arg, (dict, set)):
-            raise NotImplementedError("syntax is reserved")
-        elif arg is Ellipsis:
-            raise NotImplementedError("syntax is reserved")
-        elif arg is None:
-            raise TypeError("None in argument list")
+        elif isinstance(arg, (bytearray, memoryview)):
+            result.append(bytes(arg))
+        elif isinstance(arg, (io.StringIO, io.BytesIO)):
+            result.append(arg.getvalue())
+        elif isinstance(arg, (list, tuple, reversed, enumerate, zip)):
+            result.extend(
+                coerce(arg, None)  # pyright: ignore[reportUnknownArgumentType]
+            )
+        elif isinstance(
+            arg,
+            (
+                collections.abc.Mapping,
+                collections.abc.Set,
+                collections.abc.Generator,
+                range,
+                type(None),
+                type(Ellipsis),
+            ),
+        ):
+            type_name = type(arg).__name__  # pyright: ignore[reportUnknownArgumentType]
+            raise TypeError(f"Type {type_name!r} is not supported")
         else:
             result.append(str(arg))
     return tuple(result)
