@@ -6,7 +6,7 @@ from dataclasses import dataclass
 from typing import Optional, Union
 
 import shellous
-from shellous.util import decode
+from shellous.util import decode_bytes
 
 # Limit number of bytes of stderr stored in Result object.
 RESULT_STDERR_LIMIT = 1024
@@ -46,12 +46,12 @@ class Result:
     @property
     def output(self) -> str:
         "Output of command as a string."
-        return decode(self.output_bytes, self.encoding)
+        return decode_bytes(self.output_bytes, self.encoding)
 
     @property
     def error(self) -> str:
         "Error from command as a string (if it is not redirected)."
-        return decode(self.error_bytes, self.encoding)
+        return decode_bytes(self.error_bytes, self.encoding)
 
     def __bool__(self) -> bool:
         "Return true if exit_code is 0."
@@ -83,7 +83,7 @@ def convert_result_list(
     return Result(
         exit_code=key_result.exit_code,
         output_bytes=last.output_bytes,
-        error_bytes=last.error_bytes,
+        error_bytes=key_result.error_bytes,
         cancelled=cancelled,
         encoding=last.encoding,
     )
