@@ -13,7 +13,7 @@ import shellous
 import shellous.redirect as redir
 from shellous import pty_util
 from shellous.harvest import harvest, harvest_results
-from shellous.log import LOG_DETAIL, LOG_ENTER, LOG_EXIT, LOGGER, log_method, log_timer
+from shellous.log import LOG_DETAIL, LOGGER, log_method, log_timer
 from shellous.redirect import Redirect
 from shellous.result import (
     RESULT_STDERR_LIMIT,
@@ -25,6 +25,7 @@ from shellous.util import (
     BSD_DERIVED,
     SupportsClose,
     close_fds,
+    encode_bytes,
     poll_wait_pid,
     uninterrupted,
     verify_dev_fd,
@@ -36,14 +37,13 @@ _KILL_TIMEOUT = 3.0
 _CLOSE_TIMEOUT = 0.25
 _UNKNOWN_EXIT_CODE = 255
 
-AUDIT_EVENT_SUBPROCESS_SPAWN = "byllyfish/shellous.subprocess_spawn"
+EVENT_SHELLOUS_EXEC = "shellous.exec"
 """Audit event raised by sys.audit() when shellous runs a subprocess.
-
 The audit event has one argument: the name of the command.
 """
 
-UNLAUNCHED_EXIT_CODE = -255
-"""Special exit code used in audit events when process launch itself
+CANCELLED_EXIT_CODE = -1000
+"""Special exit code used in audit callbacks when the process launch itself
 was cancelled."""
 
 
@@ -62,11 +62,6 @@ def _is_writable(cmd: "Union[shellous.Command[Any], shellous.Pipeline[Any]]"):
     return cmd.options._writable
 
 
-def _enc(encoding: str) -> list[str]:
-    "Helper function to split the encoding name into codec and errors."
-    return encoding.split(maxsplit=1)
-
-
 class _RunOptions:
     """_RunOptions is context manager to assist in running a command.
 
@@ -84,7 +79,7 @@ class _RunOptions:
     encoding: str
     open_fds: list[Union[int, SupportsClose]]
     input_bytes: Optional[bytes]
-    pos_args: list[Union[str, bytes]]
+    pos_args: list[Union[str, bytes, os.PathLike[Any]]]
     kwd_args: dict[str, Any]
     subcmds: "list[Union[shellous.Command[Any], shellous.Pipeline[Any]]]"
     pty_fds: Optional[pty_util.PtyFds]
@@ -112,7 +107,9 @@ class _RunOptions:
             if _uses_process_substitution(self.command):
                 self.pos_args = self._setup_process_substitution()
             else:
-                self.pos_args = cast(list[Union[str, bytes]], list(self.command.args))
+                self.pos_args = cast(
+                    list[Union[str, bytes, os.PathLike[Any]]], list(self.command.args)
+                )
 
             self._setup_redirects()
             self._setup_pass_fds()
@@ -153,7 +150,7 @@ class _RunOptions:
             for subcmd in self.subcmds:
                 _cleanup(subcmd)
 
-    def _setup_process_substitution(self) -> list[Union[str, bytes]]:
+    def _setup_process_substitution(self) -> list[Union[str, bytes, os.PathLike[Any]]]:
         """Set up process substitution.
 
         Returns new command line arguments with /dev/fd path substitutions.
@@ -161,7 +158,7 @@ class _RunOptions:
         if sys.platform == "win32":
             raise RuntimeError("process substitution not supported")  # pragma: no cover
 
-        new_args: list[Union[str, bytes]] = []
+        new_args: list[Union[str, bytes, os.PathLike[Any]]] = []
         new_fds: list[int] = []
 
         for arg in self.command.args:
@@ -241,7 +238,7 @@ class _RunOptions:
             stdin=stdin,
             stdout=stdout,
             stderr=stderr,
-            env=options.merge_env(),
+            env=options.runtime_env(),
             start_new_session=start_session,
             preexec_fn=preexec_fn,
         )
@@ -288,7 +285,7 @@ class _RunOptions:
             if close:
                 self.open_fds.append(stdin)
         elif isinstance(input_, str):
-            input_bytes = input_.encode(*_enc(encoding))
+            input_bytes = encode_bytes(input_, encoding)
         elif isinstance(input_, (asyncio.StreamReader, io.BytesIO, io.StringIO)):
             # Shellous-supported input classes.
             assert stdin == asyncio.subprocess.PIPE
@@ -306,7 +303,9 @@ class _RunOptions:
 
         if isinstance(output, os.PathLike):
             mode = "ab" if append else "wb"
-            stdout = open(cast(os.PathLike[str], output), mode=mode)
+            stdout = open(  # pylint: disable=consider-using-with
+                cast(os.PathLike[str], output), mode=mode
+            )
             self.open_fds.append(stdout)
         elif self.is_stderr_only and sys_stream == sys.stderr:
             assert output == Redirect.STDOUT
@@ -388,8 +387,8 @@ class Runner:
     """Runner is an asynchronous context manager that runs a command.
 
     ```
-    async with cmd.run() as run:
-        # process run.stdin, run.stdout, run.stderr (if not None)
+    async with Runner(cmd) as run:
+        # process streams: run.stdin, run.stdout, run.stderr (if not None)
     result = run.result()
     ```
     """
@@ -438,7 +437,7 @@ class Runner:
         if not self._proc:
             if self._cancelled:
                 # The process was cancelled before starting.
-                return UNLAUNCHED_EXIT_CODE
+                return CANCELLED_EXIT_CODE
             return None
         code = self._proc.returncode
         if code == _UNKNOWN_EXIT_CODE and self._last_signal is not None:
@@ -566,7 +565,8 @@ class Runner:
         "Send a signal to the process."
         assert self._proc is not None  # (pyright)
 
-        LOGGER.debug("Runner.signal %r signal=%r", self, sig)
+        if LOG_DETAIL:
+            LOGGER.debug("Runner.signal %r signal=%r", self, sig)
         self._audit_callback("signal", signal=sig)
 
         if sig is None:
@@ -595,7 +595,7 @@ class Runner:
                 LOGGER.error("%r failed to kill process %r", self, self._proc)
                 raise RuntimeError(f"Unable to kill process {self._proc!r}") from ex
 
-    @log_method(LOG_ENTER)
+    @log_method(LOG_DETAIL)
     async def __aenter__(self):
         "Set up redirections and launch subprocess."
         self._audit_callback("start")
@@ -708,7 +708,7 @@ class Runner:
     async def _subprocess_exec(self, opts: _RunOptions):
         "Start the subprocess and assign to `self.proc`."
         with log_timer("asyncio.create_subprocess_exec"):
-            sys.audit(AUDIT_EVENT_SUBPROCESS_SPAWN, opts.pos_args[0])
+            sys.audit(EVENT_SHELLOUS_EXEC, opts.pos_args[0])
             with pty_util.set_ignore_child_watcher(
                 BSD_DERIVED and opts.pty_fds is not None
             ):
@@ -781,7 +781,7 @@ class Runner:
             return None
 
         if isinstance(source, io.StringIO):
-            input_bytes = source.getvalue().encode(*_enc(opts.encoding))
+            input_bytes = encode_bytes(source.getvalue(), opts.encoding)
             self.add_task(redir.write_stream(input_bytes, stream, eof), tag)
             return None
 
@@ -822,7 +822,7 @@ class Runner:
 
         return stream
 
-    @log_method(LOG_EXIT)
+    @log_method(LOG_DETAIL)
     async def __aexit__(
         self,
         _exc_type: Union[type[BaseException], None],
@@ -1038,7 +1038,7 @@ class PipeRunner:
             self._cancelled = True
         self._tasks.clear()  # clear all tasks when done
 
-    @log_method(LOG_ENTER)
+    @log_method(LOG_DETAIL)
     async def __aenter__(self):
         "Set up redirections and launch pipeline."
         try:
@@ -1050,7 +1050,7 @@ class PipeRunner:
             await self._wait(kill=True)
             raise
 
-    @log_method(LOG_EXIT)
+    @log_method(LOG_DETAIL)
     async def __aexit__(
         self,
         _exc_type: Union[type[BaseException], None],
