@@ -22,38 +22,31 @@ from typing import (
 from .log import LOG_DETAIL, LOGGER, log_timer
 
 _T = TypeVar("_T")
+_ContextKey = tuple[int, int]
 
 # Stores current stack of context managers for immutable Command objects.
-_CTXT_STACK = contextvars.ContextVar[
-    Optional[dict[int, list[AsyncContextManager[_T]]]]
-](
-    "ctxt_stack",
-    default=None,
-)
+_CONTEXT_STACKS = contextvars.ContextVar[
+    dict[_ContextKey, list[AsyncContextManager[_T]]]
+]("shellous.context_stacks")
 
 # True if OS is derived from BSD.
 BSD_FREEBSD = sys.platform.startswith("freebsd")
 BSD_DERIVED = BSD_FREEBSD or sys.platform == "darwin"
 
 
-def decode(data: bytes, encoding: str) -> str:
+def decode_bytes(data: bytes, encoding: str) -> str:
     "Utility function to decode byte strings."
     return data.decode(*encoding.split(maxsplit=1))
 
 
+def encode_bytes(data: str, encoding: str) -> bytes:
+    "Utility function to encode byte strings."
+    return data.encode(*encoding.split(maxsplit=1))
+
+
 def coerce_env(env: dict[str, Any]) -> dict[str, str]:
-    """Utility function to coerce environment variables to string.
-
-    If the value of an environment variable is `...`, grab the value from the
-    parent environment.
-    """
-
-    def _coerce(key: str, value: Any):
-        if value is ...:
-            value = os.environ[key]
-        return str(value)
-
-    return {str(key): _coerce(key, value) for key, value in env.items()}
+    """Utility function to coerce environment variables to string."""
+    return {str(key): str(value) for key, value in env.items()}
 
 
 class SupportsClose(Protocol):
@@ -103,7 +96,7 @@ def wait_pid(pid: int, *, block: bool = False) -> Optional[int]:
         result_pid, status = os.waitpid(pid, options)
     except ChildProcessError as ex:
         # Set status to 255 if process not found.
-        LOGGER.warning("wait_pid(%r) status is 255 ex=%r", pid, ex)
+        LOGGER.warning("wait_pid(%r) status not found (255) ex=%r", pid, ex)
         return 255
 
     if LOG_DETAIL:
@@ -133,11 +126,12 @@ def poll_wait_pid(proc: Process) -> bool:
     if status is None:
         return False
 
-    LOGGER.debug(
-        "process %r exited with returncode %r (wait_pid)",
-        proc.pid,
-        status,
-    )
+    if LOG_DETAIL:
+        LOGGER.debug(
+            "process %r exited with returncode %r (wait_pid)",
+            proc.pid,
+            status,
+        )
 
     proc._transport._returncode = status  # type: ignore
     proc._transport._proc.returncode = status  # type: ignore
@@ -156,36 +150,38 @@ async def uninterrupted(coro: Coroutine[Any, Any, _T]) -> _T:
         raise
 
 
-async def context_aenter(scope: int, ctxt_manager: AsyncContextManager[_T]) -> _T:
+async def context_aenter(owner: object, ctxt_manager: AsyncContextManager[_T]) -> _T:
     "Enter an async context manager."
-    ctxt_stack = _CTXT_STACK.get()
-    if ctxt_stack is None:
-        ctxt_stack = defaultdict[int, list[Any]](list)
-        _CTXT_STACK.set(ctxt_stack)
-
     result = await ctxt_manager.__aenter__()  # pylint: disable=unnecessary-dunder-call
-    stack = ctxt_stack[scope]
+
+    ctxt_stacks = _CONTEXT_STACKS.get(None)
+    if ctxt_stacks is None:
+        ctxt_stacks = defaultdict[_ContextKey, list[Any]](list)
+        _CONTEXT_STACKS.set(ctxt_stacks)
+
+    key = (id(owner), id(asyncio.current_task()))
+    stack = ctxt_stacks[key]
     stack.append(ctxt_manager)
 
     return result
 
 
 async def context_aexit(
-    scope: int,
+    owner: object,
     exc_type: Optional[type[BaseException]],
     exc_value: Optional[BaseException],
     exc_tb: Optional[TracebackType],
 ) -> Optional[bool]:
     "Exit an async context manager."
-    ctxt_stack = _CTXT_STACK.get()
-    assert ctxt_stack is not None  # (pyright)
-
-    stack = ctxt_stack[scope]
-    ctxt_manager = stack.pop()
-    if not stack:
-        del ctxt_stack[scope]
-    if not ctxt_stack:
-        _CTXT_STACK.set(None)
+    try:
+        ctxt_stacks = _CONTEXT_STACKS.get()
+        key = (id(owner), id(asyncio.current_task()))
+        stack = ctxt_stacks[key]
+        ctxt_manager = stack.pop()
+        if not stack:
+            del ctxt_stacks[key]
+    except Exception as ex:
+        raise RuntimeError("context var `shellous.context_stacks` is corrupt") from ex
 
     return await ctxt_manager.__aexit__(exc_type, exc_value, exc_tb)
 
